/*
Quake PSP port.
Copyright (C) 2005 Peter Mackay

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <map>

#include <pspctrl.h>

extern "C"
{
#include "../quakedef.h"
}

// A map from button mask to Quake key.
typedef std::map<unsigned int, int> ButtonToKeyMap;
static ButtonToKeyMap	buttonToGameKeyMap;
static ButtonToKeyMap	buttonToConsoleKeyMap;
static ButtonToKeyMap	buttonToMessageKeyMap;
static ButtonToKeyMap	buttonToMenuKeyMap;

// The previous key state (for checking if things changed).
static SceCtrlData		lastPad;
static bool				readyToBindKeys	= false;

void IN_Init (void)
{
	// Set up the controller.
	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

	// Japanese users would prefer to have X as cancel and O as OK.
	unsigned int	okButton		= PSP_CTRL_CROSS;
	unsigned int	cancelButton	= PSP_CTRL_CIRCLE;
	if (0)
	{
		okButton		= PSP_CTRL_CIRCLE;
		cancelButton	= PSP_CTRL_CROSS;
	}

	// Build the button to Quake key maps.
	// Common keys:
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_SELECT,		'~'));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_START,		K_ESCAPE));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_UP,			K_UPARROW));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_RIGHT,		K_RIGHTARROW));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_DOWN,			K_DOWNARROW));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_LEFT,			K_LEFTARROW));
	buttonToConsoleKeyMap	= buttonToGameKeyMap;
	buttonToMessageKeyMap	= buttonToGameKeyMap;
	buttonToMenuKeyMap		= buttonToGameKeyMap;

	// Game keys:
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_LTRIGGER,		K_AUX1));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_RTRIGGER,		K_AUX2));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_TRIANGLE,		K_JOY1));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_CIRCLE,		K_JOY2));	
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_CROSS,		K_JOY3));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_SQUARE,		K_JOY4));

	// Console keys:
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_LTRIGGER,		K_PGUP));
	buttonToGameKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_RTRIGGER,		K_PGDN));
	buttonToConsoleKeyMap.insert(ButtonToKeyMap::value_type(okButton,			K_ENTER));
	buttonToConsoleKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_SQUARE,	K_BACKSPACE));

	// Message keys:
	buttonToMessageKeyMap = buttonToConsoleKeyMap;

	// Menu keys:
	buttonToMenuKeyMap.insert(ButtonToKeyMap::value_type(cancelButton,			K_ESCAPE));
	buttonToMenuKeyMap.insert(ButtonToKeyMap::value_type(okButton,				K_ENTER));
	buttonToMenuKeyMap.insert(ButtonToKeyMap::value_type(PSP_CTRL_SQUARE,		K_BACKSPACE));
}

void IN_Shutdown (void)
{
}

void IN_Commands (void)
{
	// Changed in or out of key binding mode?
	if ((bind_grab != 0) != readyToBindKeys)
	{
		// Was in key binding mode?
		if (readyToBindKeys)
		{
			// Just left key binding mode.
			// Release all keys which are pressed.
			for (unsigned int button = 1; button != 0; button <<= 1)
			{
				// Was the button pressed?
				if (lastPad.Buttons & button)
				{
					// Is the button in the map?
					ButtonToKeyMap::const_iterator found = buttonToGameKeyMap.find(button);
					if (found != buttonToGameKeyMap.end())
					{
						// Send a release event.
						Key_Event(found->second, qfalse);
					}
				}
			}

			// We're not ready to bind keys any more.
			readyToBindKeys = false;
		}
		else
		{
			// Entering key binding mode.
			// Release all keys which are pressed.
			for (unsigned int button = 1; button != 0; button <<= 1)
			{
				// Was the button pressed?
				if (lastPad.Buttons & button)
				{
					// Is the button in the map?
					ButtonToKeyMap::const_iterator found = buttonToMenuKeyMap.find(button);
					if (found != buttonToMenuKeyMap.end())
					{
						// Send a release event.
						Key_Event(found->second, qfalse);
					}
				}
			}

			// We're now ready to bind keys.
			readyToBindKeys = true;
		}
	}

	// Use a different key mapping depending on where inputs are going to go.
	const ButtonToKeyMap* buttonToKeyMap = 0;
	switch (key_dest)
	{
	case key_game:
		buttonToKeyMap = &buttonToGameKeyMap;
		break;

	case key_console:
		buttonToKeyMap = &buttonToConsoleKeyMap;
		break;

	case key_message:
		buttonToKeyMap = &buttonToMessageKeyMap;
		break;

	case key_menu:
		// Binding keys?
		if (readyToBindKeys)
		{
			buttonToKeyMap = &buttonToGameKeyMap;
		}
		else
		{
			buttonToKeyMap = &buttonToMenuKeyMap;
		}
		break;

	default:
		Sys_Error("Unhandled key destination %d", key_dest);
		return;
	}

	// Read the pad state.
	SceCtrlData pad;
	sceCtrlReadBufferPositive(&pad, 1);

	// Find out which buttons have changed.
	SceCtrlData deltaPad;
	deltaPad.Buttons	= pad.Buttons ^ lastPad.Buttons;
	deltaPad.Lx			= pad.Lx - lastPad.Lx;
	deltaPad.Ly			= pad.Ly - lastPad.Ly;
	deltaPad.TimeStamp	= pad.TimeStamp	- lastPad.TimeStamp;

	// Handle buttons which have changed.
	for (unsigned int button = 1; button != 0; button <<= 1)
	{
		// Has the button changed?
		if (deltaPad.Buttons & button)
		{
			// Is the button in the map?
			ButtonToKeyMap::const_iterator found = buttonToKeyMap->find(button);
			if (found != buttonToKeyMap->end())
			{
				// Send an event.
				const int		key		= found->second;
				const qboolean	state	= (pad.Buttons & button) ? qtrue : qfalse;
				Key_Event(key, state);
			}
		}
	}

	// Remember the pad state for next time.
	lastPad = pad;
}

void IN_Move (usercmd_t *cmd)
{
	// Don't let the pitch drift back to centre if analog nub look is on.
	if (in_mlook.state & 1)
		V_StopPitchDrift();

	// Read the pad state.
	SceCtrlData pad;
	sceCtrlReadBufferPositive(&pad, 1);

	// Convert the inputs to floats in the range [-1, 1].
	float x = (pad.Lx - 128.0f) / 128.0f;
	float y = (pad.Ly - 128.0f) / 128.0f;

	// Implement the dead zone.
	const float deadZone	= 0.15f;
	const float deadZoneSq	= (deadZone * deadZone);
	if (((x * x) + (y * y)) < deadZoneSq)
	{
		// The joystick is inside the dead zone.
		return;
	}

	// Set the yaw.
	const float yawScale = sensitivity.value * 30.0f;
	cl.viewangles[YAW] -= yawScale * x * host_frametime;

	// Analog nub look?
	if (in_mlook.state & 1)
	{
		// Set the pitch.
		const bool invertPitch = m_pitch.value < 0;
		const float pitchScale = yawScale * 0.5f * (invertPitch ? -1 : 1);
		cl.viewangles[PITCH] += pitchScale * y * host_frametime;

		// Don't look too far up or down.
		if (cl.viewangles[PITCH] > 80.0f)
			cl.viewangles[PITCH] = 80.0f;
		if (cl.viewangles[PITCH] < -70.0f)
			cl.viewangles[PITCH] = -70.0f;
	}
	else
	{
		// Move using up and down.
		cmd->forwardmove -= cl_forwardspeed.value * y;
	}
}
