#include <vector>
/*
Quake PSP port.
Copyright (C) 2005 Peter Mackay

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <time.h>

#include <pspdebug.h>
#include <pspkernel.h>
#include <pspmoduleinfo.h>
#include <psppower.h>

extern "C"
{
#include "../quakedef.h"
}

PSP_MODULE_INFO("Quake", 0x1000, 1, 0);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

// Running a dedicated server?
qboolean			isDedicated	= qfalse;

// How big a heap to allocate.
static const size_t	heapSize	= 16 * 1024 * 1024;

// Should the main loop stop running?
static bool			quit		= false;

static int exitCallback(int arg1, int arg2, void *common)
{
	quit = true;
	return 0;
}

static int callbackThread(SceSize args, void *argp)
{
	const int cbid = sceKernelCreateCallback("exitCallback", exitCallback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();

	return 0;
}

static int setUpExitCallback(void)
{
	const int thid = sceKernelCreateThread("callbackThread", callbackThread, 0x11, 0xFA0, 0, 0);
	if (thid >= 0)
		sceKernelStartThread(thid, 0, 0);
	return thid;
}

static void disableFloatingPointExceptions()
{
	asm(
		".set push\n"
		".set noreorder\n"
		"cfc1    $2, $31\n"		// Get the FP Status Register.
		"lui     $8, 0x80\n"	// (?)
		"and     $8, $2, $8\n"	// Mask off all bits except for 23 of FCR. (? - where does the 0x80 come in?)
		"ctc1    $8, $31\n"		// Set the FP Status Register.
		".set pop\n"
		:						// No inputs.
		:						// No outputs.
		: "$2", "$8"			// Clobbered registers.
		);
}

int main(int argc, char *argv[])
{
	// Initialise the debug screen.
	pspDebugScreenInit();

	// Set up the exit callback.
	setUpExitCallback();

	// Disable floating point exceptions.
	// If this isn't done, Quake crashes from (presumably) divide by zero
	// operations.
	disableFloatingPointExceptions();

	// Increase the clock speed.
	scePowerSetClockFrequency(333, 333, 166);

	// Initialise the Common module.
#if 0
	char* myArgs[] =
	{
		"Quake", "-condebug"
	};
	COM_InitArgv(sizeof(myArgs) / sizeof(myArgs[0]), myArgs);
#else
	COM_InitArgv(argc, argv);
#endif

	// Allocate the heap.
	std::vector<unsigned char> heap(heapSize);

	// Initialise the Host module.
	quakeparms_t parms;
	memset(&parms, 0, sizeof(parms));
	parms.argc		= com_argc;
	parms.argv		= com_argv;
	parms.basedir	= ".";
	parms.memsize	= heap.size();
	parms.membase	= &heap.at(0);
	Host_Init(&parms);

	// Enter the main loop.
	clock_t lastTicks = clock();
	while(!quit)
	{
		const clock_t ticks = clock();
		const clock_t deltaTicks = ticks - lastTicks;
		const float deltaSeconds = deltaTicks * (1.0f / CLOCKS_PER_SEC);
		Host_Frame(deltaSeconds);
		lastTicks = ticks;
	}

	// Save the configs.
	Host_Shutdown();

	// Quit.
	sceKernelExitGame();
	return 0;
}
