/*
Quake PSP port.
Copyright (C) 2005 Peter Mackay

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <vector>

#include <pspdisplay.h>
#include <psptypes.h>

extern "C"
{
#include "../quakedef.h"
#include "../d_local.h"
}

typedef std::vector<unsigned char> SurfaceCache;

static const size_t				screenWidth		= 480;
static const size_t				screenHeight	= 272;
static const size_t				paletteSize		= 256;
unsigned short					d_8to16table[paletteSize];
static ScePspRGBA8888			palette[paletteSize];
static unsigned char			backBuffer[screenWidth * screenHeight];
static short					zbuffer[screenWidth * screenHeight];
static SurfaceCache				surfaceCache;

void VID_SetPalette (unsigned char *palette)
{
	for(size_t color = 0; color < paletteSize; ++color)
	{
		::palette[color] =
			palette[3 * color] |
			palette[3 * color + 1] << 8 |
			palette[3 * color + 2] << 16;
	}
}

void VID_ShiftPalette (unsigned char *palette)
{
	VID_SetPalette(palette);
}

void VID_Init (unsigned char *palette)
{
//	sceDisplaySetMode(0, SCREEN_WIDTH, SCREEN_HEIGHT);
//	sceDisplaySetFrameBuf(vram, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, PSP_DISPLAY_SETBUF_IMMEDIATE);

	vid.aspect			= 1.0f;
	vid.buffer			= backBuffer;
	vid.colormap		= host_colormap;
	vid.colormap16		= 0;
	vid.conbuffer		= backBuffer;
	vid.conheight		= screenHeight;
	vid.conrowbytes		= screenWidth;
	vid.conwidth		= screenWidth;
	vid.direct			= backBuffer;
	vid.fullbright		= 256 - LittleLong(*((int *) vid.colormap + 2048));
	vid.height			= screenHeight;
	vid.maxwarpheight	= screenWidth;
	vid.maxwarpwidth	= WARP_HEIGHT;
	vid.numpages		= 1;
	vid.recalc_refdef	= 0;
	vid.rowbytes		= screenWidth;
	vid.width			= screenWidth;
	
	d_pzbuffer = zbuffer;

	surfaceCache.resize(D_SurfaceCacheForRes(screenWidth, screenHeight));
	D_InitCaches (&surfaceCache.at(0), surfaceCache.size());

	VID_SetPalette(palette);

	// This is an awful hack to get around a division by zero later on...
	r_refdef.vrect.width	= screenWidth;
	r_refdef.vrect.height	= screenHeight;
}

void VID_Shutdown (void)
{
}

void VID_Update (vrect_t *rects)
{
	// Constants.
	ScePspRGBA8888* const	vram	= reinterpret_cast<ScePspRGBA8888*> (0x04000000);

	// For each rectangle...
	for (const vrect_t* rect = rects; rect; rect = rect->pnext)
	{
		// Set up this rectangle.
		const unsigned int		srcPitch		= vid.rowbytes;
		const pixel_t* const	srcRectBegin	= vid.buffer + (rect->y * srcPitch) + rect->x;
		const pixel_t* const	srcRectEnd		= srcRectBegin + (rect->height * srcPitch);
		const pixel_t* 			srcRowBegin		= srcRectBegin;
		const pixel_t* 			srcRowEnd		= srcRowBegin + rect->width;
		ScePspRGBA8888*			dstRowBegin		= vram + (rect->y * 512) + rect->x;

		// For each row...
		while (srcRowBegin != srcRectEnd)
		{
			// Set up this row.
			const pixel_t*	srcPixel	= srcRowBegin;
			ScePspRGBA8888*	dstPixel	= dstRowBegin;

			// For each pixel.
			while (srcPixel != srcRowEnd)
			{
				// Copy the pixel using the palette.
				*dstPixel++ = palette[*srcPixel++];
			}

			// Next row.
			srcRowBegin += srcPitch;
			srcRowEnd	+= srcPitch;
			dstRowBegin += 512;
		}
	}
}

void D_BeginDirectRect (int x, int y, byte *pbitmap, int width, int height)
{
}

void D_EndDirectRect (int x, int y, int width, int height)
{
}
